@@ -3,7 +3,7 @@
 # SPDX-License-Identifier: EUPL-1.2
 
 import logging as log
-import statsd
+import statsd  # type: ignore
 
 
 class Base:
@@ -13,10 +13,11 @@ class Base:
     properties.
     """
 
-    __slots__ = ["_logger", "_statsd"]
+    _logger: log.Logger
+    _statsd: statsd.Client
 
     @property
-    def logger(self):
+    def logger(self) -> log.Logger:
         """Return this object's logger instance, create one if necessary
 
         :return: A logger object for this instance
@@ -30,7 +31,7 @@ class Base:
         return self._logger
 
     @property
-    def statsd(self):
+    def statsd(self) -> statsd.Client:
         """Return this object's statsd instance, create one if necessary
 
         :return: A statsd object for this instance
