@@ -33,13 +33,9 @@ class RedisStore(ConfigStoreBase):
 
         super().retrieve(name)
 
-        with self.statsd.get_timer().time("retrieve_time"):
-            config = self.redis.get("saas:instance:" + name)
-
-        self.statsd.get_counter().increment("retrieve")
+        config = self.redis.get("saas:instance:" + name)
 
         if config is None:
-            self.statsd.get_counter().increment("retrieve_miss")
             raise ConfigurationNotFound
 
         parsed = self.parser.parse(config.decode("utf-8"))
