@@ -2,15 +2,25 @@
 #
 # SPDX-License-Identifier: EUPL-1.2
 
-import statsd
+import statsd  # type: ignore
 from .. import Base
 from ..infrastructure import InfrastructureFactory
-from ..repository import RepositoryFactory
+from ..repository import RepositoryBase, RepositoryFactory
 from .events import Event, EventService, event
 from abc import ABC, abstractmethod
 from copy import copy
 from functools import partial
-from typing import Any, Dict, Set, Type, cast
+from typing import (
+    Any,
+    Dict,
+    Iterable,
+    List,
+    Protocol,
+    Set,
+    Type,
+    TypedDict,
+    cast,
+)
 from uuid import UUID
 
 __all__ = [
@@ -26,7 +36,6 @@ __all__ = [
     "SplitCommandBase",
     "SplitQueryBase",
     "QueryBase",
-    "QueryMiddlewareBase",
     "UserInfo",
 ]
 
@@ -52,10 +61,7 @@ class SplitCommandBase(Base, ABC):
     This version should be used when every command is split out to its own class.
     """
 
-    @property
-    @abstractmethod
-    def name(self) -> str:
-        raise NotImplementedError
+    name: str
 
     @abstractmethod
     def __call__(self) -> None:
@@ -109,10 +115,7 @@ class SplitQueryBase(Base, ABC):
     This version should be used when every query is split out to its own class.
     """
 
-    @property
-    @abstractmethod
-    def name(self) -> str:
-        raise NotImplementedError
+    name: str
 
     @abstractmethod
     def __call__(self) -> Any:
@@ -176,7 +179,7 @@ def build_command_lookup_table(
             command.name not in command_lookup_table
         ), f"Tried to register second command with name={command.name}"
 
-        command_lookup_table[getattr(command, "name")] = command
+        command_lookup_table[command.name] = command
 
     return command_lookup_table
 
@@ -195,7 +198,7 @@ def build_query_lookup_table(
             query.name not in query_lookup_table
         ), f"Tried to register second query with name={query.name}"
 
-        query_lookup_table[getattr(query, "name")] = query
+        query_lookup_table[query.name] = query
 
     return query_lookup_table
 
@@ -250,7 +253,7 @@ class DomainQueryContainer:
                 query_instance=self,
             )
         except KeyError as e:
-            raise AttributeError from e
+            raise AttributeError(name) from e
 
 
 class CommandBase(Base, ABC):
@@ -456,9 +459,10 @@ class QueryInfrastructureCleanup(QueryMiddleware):
 class QueryWrapper(Base):
     """Wrapper class for query instances that applies middleware."""
 
-    def __init__(self, query_instance, middleware: list):
+    def __init__(self, query_instance, middleware: list, domain: str):
         self.middleware = middleware
         self.query_instance = query_instance
+        self.domain = domain
 
     def __getattr__(self, attr):
         """Get an attribute on the wrapped class, wrapped by "event" code.
@@ -479,9 +483,13 @@ class QueryWrapper(Base):
                 """Return attribute wrapped in middlewares."""
 
                 timer = cast(
-                    statsd.Timer, self.statsd.get_timer("TotalQueryTime")
+                    statsd.Timer,
+                    self.statsd.get_timer(
+                        f"query,domain={self.domain},query={attr}"
+                    ),
                 )
-                with timer.time(attr):
+
+                with timer.time():
                     wrapped_func = partial(original_attribute, *args, **kwargs)
 
                     for middleware in self.middleware:
@@ -496,9 +504,10 @@ class QueryWrapper(Base):
 class CommandWrapper(Base):
     """Wrapper class for command instances. Handles creation of Events."""
 
-    def __init__(self, command_instance):
+    def __init__(self, command_instance, domain: str):
         self.command_instance = command_instance
-        self.middlewares = []
+        self.middlewares: List[MiddlewareBase] = []
+        self.domain = domain
 
     def register_middleware(self, middleware: MiddlewareBase):
         """Register middleware to be wrapped around command.
@@ -528,10 +537,15 @@ class CommandWrapper(Base):
 
             def wrapped(*args, **kwargs):
                 """Return attribute wrapped in middlewares."""
+
                 timer = cast(
-                    statsd.Timer, self.statsd.get_timer("TotalCommandTime")
+                    statsd.Timer,
+                    self.statsd.get_timer(
+                        f"command,domain={self.domain},query={attr}"
+                    ),
                 )
-                with timer.time(attr):
+
+                with timer.time():
                     wrapped_func = partial(original_attribute, **kwargs)
                     for middleware in self.middlewares:
                         wrapped_func = partial(middleware, wrapped_func)
@@ -544,6 +558,40 @@ class CommandWrapper(Base):
             return original_attribute
 
 
+class CQRSDomain(Protocol):
+    """
+    Type for CQRS domains.
+
+    Every domain should define which repositories it has/needs, a function
+    to return a "query" instance, and one to return a "command" instance.
+    """
+
+    REQUIRED_REPOSITORIES: Dict[str, Type[RepositoryBase]]
+
+    __name__: str
+
+    @staticmethod
+    def get_query_instance(
+        repository_factory: RepositoryFactory, context: str, user_uuid: UUID
+    ) -> QueryBase:
+        ...
+
+    @staticmethod
+    def get_command_instance(
+        repository_factory: RepositoryFactory,
+        context: str,
+        user_uuid: UUID,
+        event_service: EventService,
+    ) -> CommandBase:
+        ...
+
+
+class CQRSDomainDescriptor(TypedDict):
+    module: CQRSDomain
+    repository_factory: RepositoryFactory
+    repository_factory_ro: RepositoryFactory
+
+
 class CQRS(Base):
     """Keep commands and queries separated.
 
@@ -560,7 +608,7 @@ class CQRS(Base):
 
     def __init__(
         self,
-        domains,
+        domains: Iterable[CQRSDomain],
         infrastructure_factory: InfrastructureFactory,
         command_wrapper_middleware=None,
         query_middleware=None,
@@ -579,7 +627,7 @@ class CQRS(Base):
             command.
         :type command_wrapper_middleware: list of MiddlewareBase
         """
-        self.domains: Dict[str, Dict] = {}
+        self.domains: Dict[str, CQRSDomainDescriptor] = {}
 
         if command_wrapper_middleware is None:
             command_wrapper_middleware = []
@@ -630,7 +678,7 @@ class CQRS(Base):
             }
 
     def get_query_instance(
-        self, correlation_id: UUID, domain: str, context, user_uuid: UUID
+        self, correlation_id: UUID, domain: str, context: str, user_uuid: UUID
     ):
         """Instantiate and return the "query" part of the specified domain.
 
@@ -647,8 +695,12 @@ class CQRS(Base):
             + f"'{context}' for user '{user_uuid}'"
         )
 
-        timer = cast(statsd.Timer, self.statsd.get_timer(domain))
-        with timer.time("get_query_instance"):
+        short_domain = domain.split(".")[-1]
+        timer = cast(
+            statsd.Timer,
+            self.statsd.get_timer(f"get_query_instance,domain={short_domain}"),
+        )
+        with timer.time():
             query_instance = self.domains[domain]["module"].get_query_instance(
                 self.domains[domain]["repository_factory_ro"],
                 context=context,
@@ -672,12 +724,13 @@ class CQRS(Base):
             wrapped_query = QueryWrapper(
                 query_instance=query_instance,
                 middleware=initialized_middleware,
+                domain=short_domain,
             )
 
         return wrapped_query
 
     def get_command_instance(
-        self, correlation_id: UUID, domain: str, context, user_uuid: UUID
+        self, correlation_id: UUID, domain: str, context: str, user_uuid: UUID
     ):
         """Instantiate and return the "command" instance of the specified domain.
 
@@ -699,8 +752,14 @@ class CQRS(Base):
             + f"'{context}' for user '{user_uuid}'"
         )
 
-        timer = cast(statsd.Timer, self.statsd.get_timer(domain))
-        with timer.time("get_command_instance"):
+        short_domain = domain.split(".")[-1]
+        timer = cast(
+            statsd.Timer,
+            self.statsd.get_timer(
+                f"get_command_instance,domain={short_domain}"
+            ),
+        )
+        with timer.time():
             event_service = EventService(
                 correlation_id=correlation_id,
                 domain=domain,
@@ -715,7 +774,9 @@ class CQRS(Base):
                 event_service=event_service,
             )
 
-            cmd_wrapped = CommandWrapper(command_instance=cmd_instance)
+            cmd_wrapped = CommandWrapper(
+                command_instance=cmd_instance, domain=short_domain
+            )
 
             # Always "outer layer" to clean up infrastructure
             for middleware in [
