@@ -2,13 +2,14 @@
 #
 # SPDX-License-Identifier: EUPL-1.2
 
-import magic
+import magic  # type: ignore
 import os
+import typing
 import xml.etree.ElementTree as ET
 import zipfile
 
 
-def is_zip(file_handle):
+def is_zip(file_handle: typing.BinaryIO) -> bool:
     file_handle.seek(0, os.SEEK_SET)
     header = file_handle.read(4)
 
@@ -17,7 +18,7 @@ def is_zip(file_handle):
     return False
 
 
-def get_mime_type_from_content(file_handle):
+def get_mime_type_from_content(file_handle: typing.BinaryIO) -> str:
     file_handle.seek(0, os.SEEK_SET)
 
     # This is needed to resolve an issue with the file handle
@@ -28,7 +29,7 @@ def get_mime_type_from_content(file_handle):
     return mime_type
 
 
-def get_mime_type_from_handle(file_handle):
+def get_mime_type_from_handle(file_handle: typing.BinaryIO) -> str:
     recognized_office_formats = {
         "application/vnd.openxmlformats-officedocument.wordprocessingml.document.main+xml": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
         "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet.main+xml": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
@@ -56,7 +57,7 @@ def get_mime_type_from_handle(file_handle):
         else:
             file_handle.seek(0, os.SEEK_SET)
             mime_type = get_mime_type_from_content(file_handle)
-    except (KeyError, zipfile.BadZipFile, ET.ParseError):
+    except:  # noqa (most likely: Corrupt zip file)
         file_handle.seek(0, os.SEEK_SET)
         mime_type = get_mime_type_from_content(file_handle)
 
